@@ -9,16 +9,16 @@ File::Next - File-finding iterator
 
 =head1 VERSION
 
-Version 1.17_01
+Version 1.02
 
 =cut
 
-our $VERSION = '1.17_01';
+our $VERSION = '1.02';
 
 =head1 SYNOPSIS
 
 File::Next is a lightweight, taint-safe file-finding module.
-It has no non-core prerequisites.
+It's lightweight and has no non-core prerequisites.
 
     use File::Next;
 
@@ -81,26 +81,6 @@ in I<@starting_points>.  Each call to the iterator returns another
 file, whether it's a regular file, directory, symlink, socket, or
 whatever.
 
-=head2 from_file( [ \%options, ] $filename )
-
-Returns an iterator that iterates over each of the files specified
-in I<$filename>.  If I<$filename> is C<->, then the files are read
-from STDIN.
-
-The files are assumed to be in the file one filename per line.  If
-I<$nul_separated> is passed, then the files are assumed to be
-NUL-separated, as by C<find -print0>.
-
-If there are blank lines or empty filenames in the input stream,
-they are ignored.
-
-Each filename is checked to see that it is a regular file or a named
-pipe.  If the file does not exists or is a directory, then a warning
-is thrown to I<warning_handler>, and the file is skipped.
-
-The following options have no effect in C<from_files>: I<descend_filter>,
-I<sort_files>, I<follow_symlinks>.
-
 =head1 SUPPORT FUNCTIONS
 
 =head2 sort_standard( $a, $b )
@@ -181,7 +161,7 @@ a collection of variables.
 =back
 
 The descend filter is NOT applied to any directory names specified
-as I<@starting_points> in the constructor.  For example,
+in as I<@starting_points> in the constructor.  For example,
 
     my $iter = File::Next::files( { descend_filter => sub{0} }, '/tmp' );
 
@@ -192,15 +172,7 @@ By default, the I<descend_filter> is C<sub {1}>, or "always descend".
 =head2 error_handler => \&error_handler
 
 If I<error_handler> is set, then any errors will be sent through
-it.  If the error is OS-related (ex. file not found, not permissions), the
-native error code is passed as a second argument.  By default, this value is
-C<CORE::die>.  This function must NOT return.
-
-=head2 warning_handler => \&warning_handler
-
-If I<warning_handler> is set, then any errors will be sent through
-it.  By default, this value is C<CORE::warn>.  Unlike the
-I<error_handler>, this function must return.
+it.  By default, this value is C<CORE::die>.
 
 =head2 sort_files => [ 0 | 1 | \&sort_sub]
 
@@ -222,21 +194,11 @@ such as Windows.  By default, this is true.
 Note that this filter does not apply to any of the I<@starting_points>
 passed in to the constructor.
 
-You should not set C<< follow_symlinks => 0 >> unless you specifically
-need that behavior.  Setting C<< follow_symlinks => 0 >> can be a
-speed hit, because File::Next must check to see if the file or
-directory you're about to follow is actually a symlink.
-
-=head2 nul_separated => [ 0 | 1 ]
-
-Used by the C<from_file> iterator.  Specifies that the files
-listed in the input file are separated by NUL characters, as from
-the C<find> command with the C<-print0> argument.
-
 =cut
 
 use File::Spec ();
 
+## no critic (ProhibitPackageVars)
 our $name; # name of the current file
 our $dir;  # dir of the current file
 
@@ -247,35 +209,31 @@ BEGIN {
     %files_defaults = (
         file_filter     => undef,
         descend_filter  => undef,
-        error_handler   => sub { CORE::die $_[0] },
-        warning_handler => sub { CORE::warn @_ },
+        error_handler   => sub { CORE::die @_ },
         sort_files      => undef,
         follow_symlinks => 1,
-        nul_separated   => 0,
     );
     %skip_dirs = map {($_,1)} (File::Spec->curdir, File::Spec->updir);
 }
 
 
 sub files {
-    die _bad_invocation() if @_ && defined($_[0]) && ($_[0] eq __PACKAGE__);
-
     my ($parms,@queue) = _setup( \%files_defaults, @_ );
-
     my $filter = $parms->{file_filter};
+
     return sub {
-        while ( my $entry = shift @queue ) {
-            my ( $dirname, $file, $fullpath, $is_dir, $is_file, $is_fifo ) = @{$entry};
-            if ( $is_file || $is_fifo ) {
+        while (@queue) {
+            my ($dir,$file,$fullpath) = splice( @queue, 0, 3 );
+            if ( -f $fullpath ) {
                 if ( $filter ) {
                     local $_ = $file;
-                    local $File::Next::dir = $dirname;
+                    local $File::Next::dir = $dir;
                     local $File::Next::name = $fullpath;
                     next if not $filter->();
                 }
-                return wantarray ? ($dirname,$file,$fullpath) : $fullpath;
+                return wantarray ? ($dir,$file,$fullpath) : $fullpath;
             }
-            if ( $is_dir ) {
+            elsif ( -d _ ) {
                 unshift( @queue, _candidate_files( $parms, $fullpath ) );
             }
         } # while
@@ -286,14 +244,12 @@ sub files {
 
 
 sub dirs {
-    die _bad_invocation() if @_ && defined($_[0]) && ($_[0] eq __PACKAGE__);
-
     my ($parms,@queue) = _setup( \%files_defaults, @_ );
 
     return sub {
-        while ( my $entry = shift @queue ) {
-            my ( undef, undef, $fullpath, $is_dir, undef, undef ) = @{$entry};
-            if ( $is_dir ) {
+        while (@queue) {
+            my (undef,undef,$fullpath) = splice( @queue, 0, 3 );
+            if ( -d $fullpath ) {
                 unshift( @queue, _candidate_files( $parms, $fullpath ) );
                 return $fullpath;
             }
@@ -305,90 +261,30 @@ sub dirs {
 
 
 sub everything {
-    die _bad_invocation() if @_ && defined($_[0]) && ($_[0] eq __PACKAGE__);
-
     my ($parms,@queue) = _setup( \%files_defaults, @_ );
-
     my $filter = $parms->{file_filter};
+
     return sub {
-        while ( my $entry = shift @queue ) {
-            my ( $dirname, $file, $fullpath, $is_dir, $is_file, $is_fifo ) = @{$entry};
-            if ( $is_dir ) {
+        while (@queue) {
+            my ($dir,$file,$fullpath) = splice( @queue, 0, 3 );
+            if ( -d $fullpath ) {
                 unshift( @queue, _candidate_files( $parms, $fullpath ) );
             }
             if ( $filter ) {
                 local $_ = $file;
-                local $File::Next::dir  = $dirname;
+                local $File::Next::dir = $dir;
                 local $File::Next::name = $fullpath;
                 next if not $filter->();
             }
-            return wantarray ? ($dirname,$file,$fullpath) : $fullpath;
+            return wantarray ? ($dir,$file,$fullpath) : $fullpath;
         } # while
 
         return;
     }; # iterator
 }
 
-sub from_file {
-    die _bad_invocation() if @_ && defined($_[0]) && ($_[0] eq __PACKAGE__);
-
-    my ($parms,@queue) = _setup( \%files_defaults, @_ );
-    my $err  = $parms->{error_handler};
-    my $warn = $parms->{warning_handler};
-
-    my $filename = $queue[0]->[1];
-
-    if ( !defined($filename) ) {
-        $err->( 'Must pass a filename to from_file()' );
-        return undef;
-    }
-
-    my $fh;
-    if ( $filename eq '-' ) {
-        $fh = \*STDIN;
-    }
-    else {
-        if ( !open( $fh, '<', $filename ) ) {
-            $err->( "Unable to open $filename: $!", $! + 0 );
-            return undef;
-        }
-    }
-
-    my $filter = $parms->{file_filter};
-    return sub {
-        local $/ = $parms->{nul_separated} ? "\x00" : $/;
-        while ( my $fullpath = <$fh> ) {
-            chomp $fullpath;
-            next unless $fullpath =~ /./;
-            if ( not ( -f $fullpath || -p _ ) ) {
-                $warn->( "$fullpath: No such file" );
-                next;
-            }
-
-            my ($volume,$dirname,$file) = File::Spec->splitpath( $fullpath );
-            if ( $filter ) {
-                local $_ = $file;
-                local $File::Next::dir  = $dirname;
-                local $File::Next::name = $fullpath;
-                next if not $filter->();
-            }
-            return wantarray ? ($dirname,$file,$fullpath) : $fullpath;
-        } # while
-        close $fh;
-
-        return;
-    }; # iterator
-}
-
-sub _bad_invocation {
-    my $good = (caller(1))[3];
-    my $bad  = $good;
-    $bad =~ s/(.+)::/$1->/;
-    return "$good must not be invoked as $bad";
-}
-
-sub sort_standard($$)   { return $_[0]->[1] cmp $_[1]->[1] } ## no critic (ProhibitSubroutinePrototypes)
-sub sort_reverse($$)    { return $_[1]->[1] cmp $_[0]->[1] } ## no critic (ProhibitSubroutinePrototypes)
+sub sort_standard($$)   { return $_[0]->[1] cmp $_[1]->[1] }; ## no critic (ProhibitSubroutinePrototypes)
+sub sort_reverse($$)    { return $_[1]->[1] cmp $_[0]->[1] }; ## no critic (ProhibitSubroutinePrototypes)
 
 sub reslash {
     my $path = shift;
@@ -413,8 +309,10 @@ I<$passed_parms> and I<$defaults>, plus the queue.
 The queue prep stuff takes the strings in I<@starting_points> and
 puts them in the format that queue needs.
 
-The C<@queue> that gets passed around is an array, with each entry an
-arrayref of $dir, $file and $fullpath.
+The C<@queue> that gets passed around is an array that has three
+elements for each of the entries in the queue: $dir, $file and
+$fullpath.  Items must be pushed and popped off the queue three at
+a time (spliced, really).
 
 =cut
 
@@ -433,7 +331,7 @@ sub _setup {
     }
 
     # Any leftover keys are bogus
-    for my $badkey ( sort keys %passed_parms ) {
+    for my $badkey ( keys %passed_parms ) {
         my $sub = (caller(1))[3];
         $parms->{error_handler}->( "Invalid option passed to $sub(): $badkey" );
     }
@@ -446,13 +344,12 @@ sub _setup {
 
     for ( @_ ) {
         my $start = reslash( $_ );
-        my $is_dir  = -d $start;
-        my $is_file = -f _;
-        my $is_fifo = (-p _) || ($start =~ m{^/dev/fd});
-        push @queue,
-            $is_dir
-                ? [ $start, undef,  $start, $is_dir, $is_file, $is_fifo ]
-                : [ undef,  $start, $start, $is_dir, $is_file, $is_fifo ];
+        if (-d $start) {
+            push @queue, ($start,undef,$start);
+        }
+        else {
+            push @queue, (undef,$start,$start);
+        }
     }
 
     return ($parms,@queue);
@@ -468,83 +365,54 @@ I<$parms> is the hashref of parms passed into File::Next constructor.
 =cut
 
 sub _candidate_files {
-    my $parms   = shift;
-    my $dirname = shift;
+    my $parms = shift;
+    my $dir = shift;
 
     my $dh;
-    if ( !opendir $dh, $dirname ) {
-        $parms->{error_handler}->( "$dirname: $!", $! + 0 );
+    if ( !opendir $dh, $dir ) {
+        $parms->{error_handler}->( "$dir: $!" );
         return;
     }
 
     my @newfiles;
     my $descend_filter = $parms->{descend_filter};
     my $follow_symlinks = $parms->{follow_symlinks};
+    my $sort_sub = $parms->{sort_files};
 
-    for my $file ( grep { !exists $skip_dirs{$_} } readdir $dh ) {
-        my $fullpath = File::Spec->catdir( $dirname, $file );
-        if ( !$follow_symlinks ) {
-            next if -l $fullpath;
-        }
-        else {
-            stat($fullpath);
-        }
-        my $is_dir  = -d _;
-        my $is_file = -f _;
-        my $is_fifo = (-p _) || ($fullpath =~ m{^/dev/fd});
+    while ( defined ( my $file = readdir $dh ) ) {
+        next if $skip_dirs{$file};
+        my $has_stat;
 
         # Only do directory checking if we have a descend_filter
+        my $fullpath = File::Spec->catdir( $dir, $file );
+        if ( !$follow_symlinks ) {
+            next if -l $fullpath;
+            $has_stat = 1;
+        }
+
         if ( $descend_filter ) {
-            if ( $is_dir ) {
+            if ( $has_stat ? (-d _) : (-d $fullpath) ) {
                 local $File::Next::dir = $fullpath;
                 local $_ = $file;
                 next if not $descend_filter->();
             }
         }
-        push @newfiles, [ $dirname, $file, $fullpath, $is_dir, $is_file, $is_fifo ];
+        if ( $sort_sub ) {
+            push( @newfiles, [ $dir, $file, $fullpath ] );
+        }
+        else {
+            push( @newfiles, $dir, $file, $fullpath );
+        }
     }
     closedir $dh;
 
-    my $sort_sub = $parms->{sort_files};
     if ( $sort_sub ) {
-        @newfiles = sort $sort_sub @newfiles;
+        return map { @{$_} } sort $sort_sub @newfiles;
     }
 
     return @newfiles;
 }
 
-
-=head1 DIAGNOSTICS
-
-=over
-
-=item C<< File::Next::files must not be invoked as File::Next->files >>
-
-=item C<< File::Next::dirs must not be invoked as File::Next->dirs >>
-
-=item C<< File::Next::everything must not be invoked as File::Next->everything >>
-
-=back
-
-The interface functions do not allow for the method invocation syntax and
-throw errors with the messages above. You can work around this limitation
-with L<UNIVERSAL/can>.
-
-    for my $file_system_feature (qw(dirs files)) {
-        my $iterator = File::Next->can($file_system_feature)->($options, $target_directory);
-        while (defined(my $name = $iterator->())) {
-            # ...
-        }
-    }
-
-=head1 SPEED TWEAKS
-
-=over 4
-
-=item * Don't set C<< follow_symlinks => 0 >> unless you need it.
-
-=back
-
 =head1 AUTHOR
 
 Andy Lester, C<< <andy at petdance.com> >>
@@ -552,9 +420,8 @@ Andy Lester, C<< <andy at petdance.com> >>
 =head1 BUGS
 
 Please report any bugs or feature requests to
-L<http://github.com/petdance/file-next/issues>.
-
-Note that File::Next does NOT use L<http://rt.cpan.org> for bug tracking.
+L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=File-Next>.  Note that
+File::Next does NOT use L<rt.cpan.org> for bug tracking.
 
 =head1 SUPPORT
 
@@ -568,7 +435,11 @@ You can also look for information at:
 
 =item * File::Next's bug queue
 
-L<http://github.com/petdance/file-next/issues>
+L<http://code.google.com/p/file-next/issues/list>
+
+=item * AnnoCPAN: Annotated CPAN documentation
+
+L<http://annocpan.org/dist/File-Next>
 
 =item * CPAN Ratings
 
@@ -578,9 +449,9 @@ L<http://cpanratings.perl.org/d/File-Next>
 
 L<http://search.cpan.org/dist/File-Next>
 
-=item * Source code repository
+=item * Subversion repository
 
-L<http://github.com/petdance/file-next/tree/master>
+L<https://file-next.googlecode.com/svn/trunk>
 
 =back
 
@@ -589,23 +460,12 @@ L<http://github.com/petdance/file-next/tree/master>
 All file-finding in this module is adapted from Mark Jason Dominus'
 marvelous I<Higher Order Perl>, page 126.
 
-Thanks to these fine contributors:
-Varadinsky,
-Paulo Custodio,
-Gerhard Poul,
-Brian Fraser,
-Todd Rinaldo,
-Bruce Woodward,
-Christopher J. Madsen,
-Bernhard Fisseni
-and Rob Hoelz.
-
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2005-2017 Andy Lester.
+Copyright 2006-2008 Andy Lester, all rights reserved.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the Artistic License version 2.0.
+This program is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
 
 =cut
 
